from django.db import models

from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic


from polymorphic import PolymorphicModel

from django_extensions.db.models import TimeStampedModel



class Question(PolymorphicModel, TimeStampedModel):
    '''
        Represents a question.
    '''

    class Meta:
        ordering = ['order']

    order = models.PositiveIntegerField(default=1,
        help_text='The render order')

    optional = models.BooleanField(default=False,
            help_text="If selected, user doesn't have to answer this question")

    text = models.TextField(blank=False,
        help_text='The question text')

    def __unicode__(self):
        return self.text


class TextQuestion(Question):
    '''
    Question with a text answer
    '''

    def get_form(self):
        '''
        Returns the form for this model
        '''
        from forms import TextQuestionForm
        return TextQuestionForm


class Choice(TimeStampedModel):
    '''
    Model to store the choices for multi choice questions
    '''

    order = models.PositiveIntegerField(default=1)

    label = models.TextField(default="")

    class Meta:
        ordering = ['order']

    def __unicode__(self):
        return self.label


class ChoiceQuestion(Question):
    '''
    Question with answers that are chosen from a list of choices defined
    by the user.
    '''

    multichoice = models.BooleanField(default=False,
            help_text="Select one or more")

    choices = models.ManyToManyField(Choice)

    def get_form(self):
        '''
        Returns the form for this model
        '''

        from forms import ChoiceQuestionForm
        return ChoiceQuestionForm


class Answer(PolymorphicModel, TimeStampedModel):
    ''' Base Answer model
    '''
    question = models.ForeignKey(Question)

    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')


class TextAnswer(Answer):
    answer = models.TextField(blank=False,
        help_text = 'The answer text')

    def __unicode__(self):
        return self.answer


class ChoiceAnswer(Answer):
    answer = models.ManyToManyField(Choice,
        help_text='The selected choices as the answer')

    def __unicode__(self):
        return unicode(self.question)

        # XXX: the following does a weird recursion depth exception o_O ?
        #''.join(self.answer.all().values_list('label', flat=True))



from django import forms

from django.db.models import Count
from django.contrib.contenttypes.models import ContentType

from crispy_forms.helper import FormHelper
from crispy_forms.bootstrap import InlineRadios, InlineCheckboxes
from crispy_forms.layout import Layout
from crispy_forms.layout import Submit


from models import TextAnswer, ChoiceAnswer, Choice

# List of validator_name:func_name
# Show in admin a multichoice list of validator names
# pass this to form using field_name='validator_name' ?


class QuestionForm(forms.Form):
    ''' Base class for a Question
    '''

    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_method = 'post'

        # If true crispy-forms will render a <form>..</form> tags
        self.helper.form_tag = kwargs.get('form_tag', True)

        if 'form_tag' in kwargs:
            del kwargs['form_tag']

        self.content_object = kwargs.get('content_object')
        if not self.content_object:
            raise ValueError('Need a content_object to save answers too')

        self.content_type = ContentType.objects.get_for_model(self.content_object)

        del kwargs['content_object']

        self.helper.form_class = kwargs.get('form_class', '')

        self.question = kwargs.get('question')

        if not self.question:
            raise ValueError('Need a question to render')

        del kwargs['question']
        super(QuestionForm, self).__init__(*args, **kwargs)




class TextQuestionForm(QuestionForm):
    def __init__(self, *args, **kwargs):
        super(TextQuestionForm, self).__init__(*args, **kwargs)

        # work out initial data

        initial_answer = TextAnswer.objects.filter(
            object_id=self.content_object.id,
            question=self.question
        )

        initial_answer = initial_answer[0].answer if initial_answer.exists() else ''

        self.fields['answer'] = forms.CharField(
            label=self.question.text,
            widget=forms.TextInput(),
            required=not self.question.optional,
            initial=initial_answer,
        )
        answer = self.fields['answer']

    def save(self):
        if not self.is_valid():
            raise forms.ValidationError('form is not valid')

        answer = self.cleaned_data.get('answer')

        if not answer:
            if self.fields['answer'].required:
                raise forms.ValidationError, 'Required'
            return

        text_answer, created = TextAnswer.objects.get_or_create(
            object_id=self.content_object.id,
            content_type=self.content_type,
            question=self.question,
            answer=answer
        )

        if created:
            text_answer.content_object = self.content_object
            text_answer.save()


class ChoiceQuestionForm(QuestionForm):
    def __init__(self, *args, **kwargs):
        super(ChoiceQuestionForm, self).__init__(*args, **kwargs)

        choices = [(c.id, c.label) for c in self.question.choices.all()]

        # initial values

        initial_choices = []
        choice_answer = ChoiceAnswer.objects.filter(
            object_id=self.content_object.id,
            question=self.question,
        ).annotate(a=Count('answer')).filter(a__gt=0)

        # we have ChoiceAnswer instance
        if choice_answer:
            choice_answer = choice_answer[0]
            initial_choices = choice_answer.answer.all().values_list('id', flat=True)
            if self.question.multichoice is False:
                initial_choices = initial_choices=[0]


        # default classes
        widget = forms.RadioSelect
        field_type = forms.ChoiceField
        inline_type = InlineRadios

        if self.question.multichoice:
            field_type = forms.MultipleChoiceField
            widget = forms.CheckboxSelectMultiple
            inline_type = InlineCheckboxes

        field = field_type(
            label=self.question.text,
            required=not self.question.optional,
            choices=choices,
            initial=initial_choices,
            widget=widget
        )

        self.fields['answer'] = field

        # Render choice buttons inline
        self.helper.layout = Layout(
            inline_type('answer')
        )


    def save(self):
        if not self.is_valid():
            raise forms.ValidationError('Form is not valid')

        real_answer = self.cleaned_data.get('answer')

        if not real_answer:
            if self.fields['answer'].required:
                raise forms.ValidationError, 'Required'
            return

        choices = Choice.objects.filter(id__in=real_answer)

        # find ChoiceAnswer and filter in answer !
        choice_answer = ChoiceAnswer.objects.filter(
            object_id=self.content_object.id,
            content_type=self.content_type,
            question=self.question,
        )

        # we have ChoiceAnswer instance
        if choice_answer:
            choice_answer = choice_answer[0]

        if not choice_answer:
            # create a ChoiceAnswer
            choice_answer  = ChoiceAnswer.objects.create(
                object_id=self.content_object.id,
                content_type=self.content_type,
                question=self.question
            )

        # re save out the choices
        choice_answer.content_object = self.content_object
        choice_answer.answer = choices
        choice_answer.save()



DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:'
    }
}


INSTALLED_APPS = [
    'django.contrib.contenttypes',
    'django_extensions',
    'south',
    'domande'
]


SECRET_KEY = 'lol im secret'



from setuptools import setup

__version__ = '0.1.2'

description = '''django app to represent generic questions on forms'''

setup(
    name='django-domande',
    packages=[
        'domande',
        'domande.migrations'
    ],
    version=__version__,
    description=description,
    author='Bulkan Evcimen',
    author_email='bulkan@gmail.com',
    url='https://github.com/bulkan/django-domande',
    install_requires=[
        'django_polymorphic',
        'south',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7'
    ]
)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Question'
        db.create_table('domande_question', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('polymorphic_ctype', self.gf('django.db.models.fields.related.ForeignKey')(related_name='polymorphic_domande.question_set', null=True, to=orm['contenttypes.ContentType'])),
            ('created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now, blank=True)),
            ('order', self.gf('django.db.models.fields.PositiveIntegerField')(default=1)),
            ('optional', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('text', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal('domande', ['Question'])

        # Adding model 'TextQuestion'
        db.create_table('domande_textquestion', (
            ('question_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['domande.Question'], unique=True, primary_key=True)),
        ))
        db.send_create_signal('domande', ['TextQuestion'])

        # Adding model 'Choice'
        db.create_table('domande_choice', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now, blank=True)),
            ('order', self.gf('django.db.models.fields.PositiveIntegerField')(default=1)),
            ('label', self.gf('django.db.models.fields.TextField')(default='')),
        ))
        db.send_create_signal('domande', ['Choice'])

        # Adding model 'ChoiceQuestion'
        db.create_table('domande_choicequestion', (
            ('question_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['domande.Question'], unique=True, primary_key=True)),
            ('multichoice', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('domande', ['ChoiceQuestion'])

        # Adding M2M table for field choices on 'ChoiceQuestion'
        db.create_table('domande_choicequestion_choices', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('choicequestion', models.ForeignKey(orm['domande.choicequestion'], null=False)),
            ('choice', models.ForeignKey(orm['domande.choice'], null=False))
        ))
        db.create_unique('domande_choicequestion_choices', ['choicequestion_id', 'choice_id'])

        # Adding model 'Answer'
        db.create_table('domande_answer', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('polymorphic_ctype', self.gf('django.db.models.fields.related.ForeignKey')(related_name='polymorphic_domande.answer_set', null=True, to=orm['contenttypes.ContentType'])),
            ('created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now, blank=True)),
            ('question', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['domande.Question'])),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'])),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal('domande', ['Answer'])

        # Adding model 'TextAnswer'
        db.create_table('domande_textanswer', (
            ('answer_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['domande.Answer'], unique=True, primary_key=True)),
            ('answer', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal('domande', ['TextAnswer'])

        # Adding model 'ChoiceAnswer'
        db.create_table('domande_choiceanswer', (
            ('answer_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['domande.Answer'], unique=True, primary_key=True)),
        ))
        db.send_create_signal('domande', ['ChoiceAnswer'])

        # Adding M2M table for field answer on 'ChoiceAnswer'
        db.create_table('domande_choiceanswer_answer', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('choiceanswer', models.ForeignKey(orm['domande.choiceanswer'], null=False)),
            ('choice', models.ForeignKey(orm['domande.choice'], null=False))
        ))
        db.create_unique('domande_choiceanswer_answer', ['choiceanswer_id', 'choice_id'])


    def backwards(self, orm):
        # Deleting model 'Question'
        db.delete_table('domande_question')

        # Deleting model 'TextQuestion'
        db.delete_table('domande_textquestion')

        # Deleting model 'Choice'
        db.delete_table('domande_choice')

        # Deleting model 'ChoiceQuestion'
        db.delete_table('domande_choicequestion')

        # Removing M2M table for field choices on 'ChoiceQuestion'
        db.delete_table('domande_choicequestion_choices')

        # Deleting model 'Answer'
        db.delete_table('domande_answer')

        # Deleting model 'TextAnswer'
        db.delete_table('domande_textanswer')

        # Deleting model 'ChoiceAnswer'
        db.delete_table('domande_choiceanswer')

        # Removing M2M table for field answer on 'ChoiceAnswer'
        db.delete_table('domande_choiceanswer_answer')


    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'domande.answer': {
            'Meta': {'object_name': 'Answer'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'polymorphic_ctype': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'polymorphic_domande.answer_set'", 'null': 'True', 'to': "orm['contenttypes.ContentType']"}),
            'question': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['domande.Question']"})
        },
        'domande.choice': {
            'Meta': {'ordering': "['order']", 'object_name': 'Choice'},
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.TextField', [], {'default': "''"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1'})
        },
        'domande.choiceanswer': {
            'Meta': {'object_name': 'ChoiceAnswer', '_ormbases': ['domande.Answer']},
            'answer': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['domande.Choice']", 'symmetrical': 'False'}),
            'answer_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['domande.Answer']", 'unique': 'True', 'primary_key': 'True'})
        },
        'domande.choicequestion': {
            'Meta': {'ordering': "['order']", 'object_name': 'ChoiceQuestion', '_ormbases': ['domande.Question']},
            'choices': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['domande.Choice']", 'symmetrical': 'False'}),
            'multichoice': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'question_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['domande.Question']", 'unique': 'True', 'primary_key': 'True'})
        },
        'domande.question': {
            'Meta': {'ordering': "['order']", 'object_name': 'Question'},
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'optional': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1'}),
            'polymorphic_ctype': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'polymorphic_domande.question_set'", 'null': 'True', 'to': "orm['contenttypes.ContentType']"}),
            'text': ('django.db.models.fields.TextField', [], {})
        },
        'domande.textanswer': {
            'Meta': {'object_name': 'TextAnswer', '_ormbases': ['domande.Answer']},
            'answer': ('django.db.models.fields.TextField', [], {}),
            'answer_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['domande.Answer']", 'unique': 'True', 'primary_key': 'True'})
        },
        'domande.textquestion': {
            'Meta': {'ordering': "['order']", 'object_name': 'TextQuestion', '_ormbases': ['domande.Question']},
            'question_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['domande.Question']", 'unique': 'True', 'primary_key': 'True'})
        }
    }

    complete_apps = ['domande']


from django.contrib import admin

from polymorphic.admin import PolymorphicParentModelAdmin, PolymorphicChildModelAdmin

from models import Question, TextQuestion, ChoiceQuestion, Choice
from models import Answer, TextAnswer, ChoiceAnswer


class QuestionChildAdmin(PolymorphicChildModelAdmin):
    """
    Base admin class for all child models of Question
    """

    base_model = Question


class TextQuestionAdmin(QuestionChildAdmin):
    """
    ModelAdmin for a TextQuestion
    """


class ChoiceQuestionAdmin(QuestionChildAdmin):
    """
    ModelAdmin for a ChoiceQuestion
    """


class QuestionParentAdmin(PolymorphicParentModelAdmin):
    """
    Question parent model admin
    """

    base_model = Question
    child_models = (
        (TextQuestion, TextQuestionAdmin),
        (ChoiceQuestion, ChoiceQuestionAdmin),
    )


admin.site.register(Question, QuestionParentAdmin)
admin.site.register(Choice)


class AnswerChildAdmin(PolymorphicChildModelAdmin):
    """
    Base admin class for all child Answer models
    """

    base_model = Answer


class TextAnswerAdmin(AnswerChildAdmin):
    """
    ModelAdmin for TextAnswer
    """


class ChoiceAnswerAdmin(AnswerChildAdmin):
    """
    ModelAdmin for ChoiceAnswer
    """


class AnswerParentAdmin(PolymorphicParentModelAdmin):
    """
    The parent model admin for answer
    """

    list_display = (
        'object_id',
        'content_object',
        'question',
    )

    base_model = Answer
    child_models = (
        (TextAnswer, TextAnswerAdmin),
        (ChoiceAnswer, ChoiceAnswerAdmin),
    )

admin.site.register(Answer, AnswerParentAdmin)
